@@ -22,12 +22,21 @@ execution to date.
 """
 
 import inspect
-from collections.abc import Iterable
 from copy import copy
 from functools import lru_cache
 from io import StringIO
 from operator import attrgetter
-from typing import Any, Dict, List
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Sequence,
+    Union,
+    overload,
+)
 from unittest import TestCase
 
 import attr
@@ -35,7 +44,7 @@ import attr
 from hypothesis import strategies as st
 from hypothesis._settings import HealthCheck, Verbosity, settings as Settings
 from hypothesis.control import current_build_context
-from hypothesis.core import given
+from hypothesis.core import TestFunc, given
 from hypothesis.errors import InvalidArgument, InvalidDefinition
 from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.reflection import function_digest, nicerepr, proxies
@@ -43,6 +52,8 @@ from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
 from hypothesis.strategies._internal.featureflags import FeatureStrategy
 from hypothesis.strategies._internal.strategies import (
+    Ex,
+    Ex_Inv,
     OneOfStrategy,
     SearchStrategy,
     check_strategy,
@@ -53,6 +64,11 @@ STATE_MACHINE_RUN_LABEL = cu.calc_label_from_name("another state machine step")
 SHOULD_CONTINUE_LABEL = cu.calc_label_from_name("should we continue drawing")
 
 
+class _OmittedArgument:
+    """Sentinel class to prevent overlapping overloads in type hints. See comments
+    above the overloads of @rule."""
+
+
 class TestCaseProperty:  # pragma: no cover
     def __get__(self, obj, typ=None):
         if obj is not None:
@@ -413,8 +429,8 @@ class BundleReferenceStrategy(SearchStrategy):
             return bundle[position]
 
 
-class Bundle(SearchStrategy):
-    def __init__(self, name, consume=False):
+class Bundle(SearchStrategy[Ex]):
+    def __init__(self, name: str, consume: bool = False) -> None:
         self.name = name
         self.__reference_strategy = BundleReferenceStrategy(name, consume)
 
@@ -441,12 +457,12 @@ class Bundle(SearchStrategy):
         return bool(machine.bundle(self.name))
 
 
-class BundleConsumer(Bundle):
-    def __init__(self, bundle):
+class BundleConsumer(Bundle[Ex]):
+    def __init__(self, bundle: Bundle[Ex]) -> None:
         super().__init__(bundle.name, consume=True)
 
 
-def consumes(bundle):
+def consumes(bundle: Bundle[Ex]) -> SearchStrategy[Ex]:
     """When introducing a rule in a RuleBasedStateMachine, this function can
     be used to mark bundles from which each value used in a step with the
     given rule should be removed. This function returns a strategy object
@@ -465,14 +481,16 @@ def consumes(bundle):
 
 
 @attr.s()
-class MultipleResults(Iterable):
+class MultipleResults(Iterable[Ex]):
     values = attr.ib()
 
     def __iter__(self):
         return iter(self.values)
 
 
-def multiple(*args):
+# We need to use an invariant typevar here to avoid a mypy error, as covariant
+# typevars cannot be used as parameters.
+def multiple(*args: Ex_Inv) -> MultipleResults[Ex_Inv]:
     """This function can be used to pass multiple results to the target(s) of
     a rule. Just use ``return multiple(result1, result2, ...)`` in your rule.
 
@@ -517,7 +535,68 @@ PRECONDITIONS_MARKER = "hypothesis_stateful_preconditions"
 INVARIANT_MARKER = "hypothesis_stateful_invariant"
 
 
-def rule(*, targets=(), target=None, **kwargs):
+_RuleType = Callable[..., Union[MultipleResults[Ex], Ex]]
+_RuleWrapper = Callable[[_RuleType[Ex]], _RuleType[Ex]]
+
+
+# We cannot exclude `target` or `targets` from any of these signatures because
+# otherwise they would be matched against the `kwargs`, either leading to
+# overlapping overloads of incompatible return types, or a concrete
+# implementation that does not accept all overloaded variant signatures.
+# Although it is possible to reorder the variants to fix the former, it will
+# always lead to the latter, as then the omitted parameter could be typed as
+# a `SearchStrategy`, which the concrete implementation does not accept.
+#
+# Omitted `targets` parameters, where the default value is used, are typed with
+# a special `_OmittedArgument` type. We cannot type them as `Tuple[()]`, because
+# `Tuple[()]` is a subtype of `Sequence[Bundle[Ex]]`, leading to signature
+# overlaps with incompatible return types. The `_OmittedArgument` type will never be
+# encountered at runtime, and exists solely to annotate the default of `targets`.
+# PEP 661 (Sentinel Values) might provide a more elegant alternative in the future.
+#
+# We could've also annotated `targets` as `Tuple[_OmittedArgument]`, but then when
+# both `target` and `targets` are provided, mypy describes the type error as an
+# invalid argument type for `targets` (expected `Tuple[_OmittedArgument]`, got ...).
+# By annotating it as a bare `_OmittedArgument` type, mypy's error will warn that
+# there is no overloaded signature matching the call, which is more descriptive.
+#
+# When `target` xor `targets` is provided, the function to decorate must return
+# a value whose type matches the one stored in the bundle. When neither are
+# provided, the function to decorate must return nothing. There is no variant
+# for providing `target` and `targets`, as these parameters are mutually exclusive.
+@overload
+def rule(
+    *,
+    targets: Sequence[Bundle[Ex]],
+    target: None = ...,
+    **kwargs: SearchStrategy,
+) -> _RuleWrapper[Ex]:
+    raise NotImplementedError
+
+
+@overload
+def rule(
+    *, target: Bundle[Ex], targets: _OmittedArgument = ..., **kwargs: SearchStrategy
+) -> _RuleWrapper[Ex]:
+    raise NotImplementedError
+
+
+@overload
+def rule(
+    *,
+    target: None = ...,
+    targets: _OmittedArgument = ...,
+    **kwargs: SearchStrategy,
+) -> Callable[[Callable[..., None]], Callable[..., None]]:
+    raise NotImplementedError
+
+
+def rule(
+    *,
+    targets: Union[Sequence[Bundle[Ex]], _OmittedArgument] = (),
+    target: Optional[Bundle[Ex]] = None,
+    **kwargs: SearchStrategy,
+) -> Union[_RuleWrapper[Ex], Callable[[Callable[..., None]], Callable[..., None]]]:
     """Decorator for RuleBasedStateMachine. Any Bundle present in ``target`` or
     ``targets`` will define where the end result of this function should go. If
     both are empty then the end result will be discarded.
@@ -571,7 +650,40 @@ def rule(*, targets=(), target=None, **kwargs):
     return accept
 
 
-def initialize(*, targets=(), target=None, **kwargs):
+# See also comments of `rule`'s overloads.
+@overload
+def initialize(
+    *,
+    targets: Sequence[Bundle[Ex]],
+    target: None = ...,
+    **kwargs: SearchStrategy,
+) -> _RuleWrapper[Ex]:
+    raise NotImplementedError
+
+
+@overload
+def initialize(
+    *, target: Bundle[Ex], targets: _OmittedArgument = ..., **kwargs: SearchStrategy
+) -> _RuleWrapper[Ex]:
+    raise NotImplementedError
+
+
+@overload
+def initialize(
+    *,
+    target: None = ...,
+    targets: _OmittedArgument = ...,
+    **kwargs: SearchStrategy,
+) -> Callable[[Callable[..., None]], Callable[..., None]]:
+    raise NotImplementedError
+
+
+def initialize(
+    *,
+    targets: Union[Sequence[Bundle[Ex]], _OmittedArgument] = (),
+    target: Optional[Bundle[Ex]] = None,
+    **kwargs: SearchStrategy,
+) -> Union[_RuleWrapper[Ex], Callable[[Callable[..., None]], Callable[..., None]]]:
     """Decorator for RuleBasedStateMachine.
 
     An initialize decorator behaves like a rule, but all ``@initialize()`` decorated
@@ -623,7 +735,9 @@ class VarReference:
     name = attr.ib()
 
 
-def precondition(precond):
+# There are multiple alternatives for annotating the `precond` type, all of them
+# have drawbacks. See https://github.com/HypothesisWorks/hypothesis/pull/3068#issuecomment-906642371
+def precondition(precond: Callable[[Any], bool]) -> Callable[[TestFunc], TestFunc]:
     """Decorator to apply a precondition for rules in a RuleBasedStateMachine.
     Specifies a precondition for a rule to be considered as a valid step in the
     state machine, which is more efficient than using :func:`~hypothesis.assume`
@@ -688,7 +802,7 @@ class Invariant:
     check_during_init = attr.ib()
 
 
-def invariant(*, check_during_init=False):
+def invariant(*, check_during_init: bool = False) -> Callable[[TestFunc], TestFunc]:
     """Decorator to apply an invariant for rules in a RuleBasedStateMachine.
     The decorated function will be run after every rule and can raise an
     exception to indicate failed invariants.
