@@ -87,6 +87,20 @@ def type_sorting_key(t):
     return (int(issubclass(t, collections.abc.Container)), repr(t))
 
 
+def _compatible_args(args, superclass_args):
+    """Check that the args of two generic types are compatible for try_issubclass."""
+    assert superclass_args is not None
+    if args is None:
+        return True
+    return len(args) == len(superclass_args) and all(
+        # "a==b or either is a typevar" is a hacky approximation, but it's
+        # good enough for all the cases that I've seen so far and has the
+        # substantial virtue of (relative) simplicity.
+        a == b or isinstance(a, typing.TypeVar) or isinstance(b, typing.TypeVar)
+        for a, b in zip(args, superclass_args)
+    )
+
+
 def try_issubclass(thing, superclass):
     try:
         # In this case we're looking at two distinct classes - which might be generics.
@@ -106,26 +120,12 @@ def try_issubclass(thing, superclass):
             # generics, and so on.  If you need to change this code, read PEP-560
             # and Hypothesis issue #2951 closely first, and good luck.  The tests
             # will help you, I hope - good luck.
-            thing_args = getattr(thing, "__args__", None)
-            thing_orig_bases = getattr(thing, "__orig_bases__", None) or [None]
-            if thing_args is None and len(thing_orig_bases) != 1:  # pragma: no cover
-                raise ResolutionFailed(
-                    f"It looks like you're trying to resolve {thing!r} which "
-                    "inherits from more than one generic type, which isn't supported. "
-                    "Try using register_type_strategy()."
-                )
-            thing_orig_bases_args = getattr(thing_orig_bases[0], "__args__", None)
-            if thing_args is None and thing_orig_bases_args is not None:
-                return len(thing_orig_bases_args) == len(superclass_args) and all(
-                    # "a==b or either is a typevar" is a hacky approximation, but it's
-                    # good enough for all the cases that I've seen so far and has the
-                    # substantial virtue of (relative) simplicity.
-                    a == b
-                    or isinstance(a, typing.TypeVar)
-                    or isinstance(b, typing.TypeVar)
-                    for a, b in zip(thing_orig_bases_args, superclass_args)
-                )
-            return True
+            if getattr(thing, "__args__", None) is not None:
+                return True
+            for orig_base in getattr(thing, "__orig_bases__", None) or [None]:
+                args = getattr(orig_base, "__args__", None)
+                if _compatible_args(args, superclass_args):
+                    return True
         return False
     except (AttributeError, TypeError):
         # Some types can't be the subject or object of an instance or subclass check
